#!/bin/bash

################################################################################
# File: injecso.sh
# Version: 1.0
# InjecSO - Shared object injection into processes under Linux
# Support script for computing the address of __libc_dlopen_mode() in a process
################################################################################
# Author: Emilien Girault (Trance)
# Websites: www.emiliengirault.fr - www.ghostsinthestack.org
# Date: 2009/02/13
################################################################################

version="1.0"

# Params checking
if [ $# -ne 2 ]; then
  echo "InjecSO - Shared object injection into processes under Linux - v$version"
  echo "by Emilien Girault (Trance) - www.emiliengirault.fr - www.ghostsinthestack.org"
  echo
  echo "Usage: $(basename $0) pid lib.so"
  echo "pid     - PID of the target process"
  echo "lib.so  - Name of the library to inject (absolute or relative)"
  exit 0
fi

# Param renaming
pid=$1
lib=$2

# Test whether the pid is numeric
expr $pid + 0 1>/dev/null 2>&1
if [ $? -ne 0 ]; then
  echo "Error: pid is not numeric."
  exit 1
fi


if [ ! -d /proc/$pid ]; then
  echo "Error: The given PID is incorrect."
  exit 1
fi


# Get the map of process and get the line that correspond to the executable section of libc
line=$(cat /proc/$pid/maps | grep "libc\b" | grep r-x | head -n 1)

if [ -z "$line" ]; then
  echo "Error: The given pid doesn't have libc.so loaded."
  exit 1
fi

# Extract the base address of that section and the name of the library
libc_baseaddr=$(echo "$line" | cut -d "-" -f1)
libc_name=$(echo "$line" | perl -n -e '/(\S+$)/ && print $1') # extract the last field

# Use readelf to find the offset of __libc_dlopen_mode
dlopen_offset=$(readelf -s -D $libc_name | grep __libc_dlopen_mode | head -n 1 | perl -n -e '/^\s*\S+\s+\S+\s+(\S+)/ && print $1') # extract the 3rd field

# Compute the actual addresses
dlopen_addr=$(expr $(printf "%d" 0x$libc_baseaddr) + $(printf "%d" 0x$dlopen_offset))
dlopen_addr_hex=$(printf "0x%x" $dlopen_addr)

echo "[+] Found __libc_dlopen_mode at $dlopen_addr_hex"
echo "[+] Launching: injecso $pid $lib $dlopen_addr_hex"

# Launch InjecSO
$(dirname $0)/injecso $pid $(pwd)/$lib $dlopen_addr_hex

